/* START_LICENSE_HEADER

Copyright (C) 2000 Martin Piper, original design and program code
Copyright (C) 2001-2005 Replica Software

This program file is copyright (C) Replica Software and can only be used under license.
For more information visit: http://www.replicanet.com/
Or email: info@replicanet.com

END_LICENSE_HEADER */
/* XPSocket.h */
#ifndef _XPSOCK_H_
#define _XPSOCK_H_

#include <string>

namespace RNReplicaNet
{

#define	_XPSOCK_VERSION 1100


/* Some error return defines */
/**
 * \def XPSOCK_EOK No error, everything is OK
 */
#define XPSOCK_EOK					(0)		  				/**< Not an error but defined for completeness */
#define XPSOCK_EERROR				(-1)					/**< An undefined error. Look in asocket_lasterror for a fuller description */
#define XPSOCK_EWOULDBLOCK			(-2)					/**< Not a serious error, just informational */
#define XPSOCK_ECLOSED				(-3)					/**< The other end has closed the connection */
#define XPSOCK_ENOEXIST				(-4)					/**< There is not a machine there to connect to */
#define XPSOCK_ENOMEMORY			(-5)					/**< The sockets ran out of memory */
#define XPSOCK_ENOTASOCKET			(-6)					/**< The socket pointer passed into a function was not a socket pointer */
#define XPSOCK_ECANTLISTEN			(-7)					/**< Couldn't make this socket listen */
#define XPSOCK_ENOTALISTEN			(-8)					/**< This socket isn't a listen socket */
#define XPSOCK_ENOTCONNECTED		(-9)					/**< This socket isn't connected to another socket */
#define XPSOCK_EBUFFEROVERFLOW		(-10)					/**< The local host buffer has been over flowed, increase the standard amount in this header file */
#define XPSOCK_EUDPSENDFAILED		(-11)					/**< Sending a UDP packet failed. As a last resort to sending a slow packet instead */
#define XPSOCK_ECANTBIND			(-12)					/**< Can't bind the socket to an address */
#define XPSOCK_ECANTRESOLVE			(-13)					/**< Can't resolve the supplied string to a number */
#define XPSOCK_EURGENTTOOBIG		(-14)					/**< The size of the urgent data is too big */
#define XPSOCK_BUFFERTOOSMALL		(-15)					/**< The size of the destination buffer is too small */
#define XPSOCK_ILLEGALOPERATION		(-16)					/**< The requested operation cannot be performed on this socket due to type differences between urgent and normal sockets */
#define XPSOCK_ECONNINPROGRESS		(-17)					/**< A connection is in progress */
#define XPSOCK_ESTACKSTARTFAILURE	(-18)					/**< The network stack failed to start */

#define pXPSOCK_EERROR  ((t_XPSocket *) XPSOCK_EERROR)	/**< Define a pointer type to check with for an error return */

/* Probable XP socket types */
#define XPSOCK_TLOCALHOST	(0)
#define XPSOCK_TINET		(1)

/* XP socket status */
#define XPSOCK_SNULL		(0)
#define XPSOCK_SLISTEN		(1)
#define XPSOCK_SCONNECT		(2)
#define XPSOCK_SCLOSE		(3)

/* Flags to pass to XPSock_Recv */
#define XPSOCK_FNULL			(0)					/**< NULL flag to pass to XPSock_Recv */
#define XPSOCK_FPEEK			(1)					/**< PEEK flag to pass to XPSock_Recv
														This returns the message but leaves it in the queue */
#define XPSOCK_FACK				(2)					/**< Marks the packet as needing an ACK request */


#define XPSOCK_PORT_ANY			(0)					/**< Port number define to allow any port to be chosen */
#define XPSOCK_PORT_LOCAL		(-1)				/**< Port number to force local loopback operation */

#define XPSOCK_MAXPACKETSIZE	(8192)				/**< Maximum size that a packet can be */

#define XPSOCK_URGENTMAXPACKETSIZE	(2048)			/**< Maximum size that a packet can be for urgent sockets */

/* Some constants for the local loop back sockets */
#define XPSOCK_LOOPBACKBUFFER	20480

/* Flags returned by XPSock_GetHardwareStatus */
#define XPSOCK_HARDWARE_AVAILABLE			(1<<0)
#define XPSOCK_HARDWARE_CABLE_CONNECTED		(1<<1)
#define XPSOCK_HARDWARE_10MBS				(1<<2)
#define XPSOCK_HARDWARE_100MBS				(1<<3)
#define XPSOCK_HARDWARE_FULL_DUPLEX			(1<<4)
#define XPSOCK_HARDWARE_HALF_DUPLEX			(1<<5)


/* Structs and their type defs */

/**
 * This is an address that the sockets use
 */
struct XPAddress
{
	/**
	 * ctor that accepts a std::string in the form xx.xx.xx.xx:xx and converts it to an address
	 */
	XPAddress(std::string addr);

	/**
	 * ctor for XPAddress that allows the structure to be constructed in a C++ fashion
	 */
	XPAddress(const int port,unsigned char addr1,unsigned char addr2,unsigned char addr3,unsigned char addr4);

	/**
	 * default ctor, all members are left alone
	 */
	XPAddress();

	/**
	 * Exports the XPAddress as a std::string
	 * \return returns the std::string version of this XPAddress
	 */
	std::string Export(void);

	int port;										/**< The port number for this address */
	unsigned char addr[4];							/**< The IP addr for this structure */
};
typedef struct XPAddress t_XPAddress;
typedef struct XPAddress XPAddress;

/* A useful operator to compare addresses */
inline bool operator== (const t_XPAddress& a,const t_XPAddress& b)
{
	return (a.addr[0] == b.addr[0]) && (a.addr[1] == b.addr[1]) && (a.addr[2] == b.addr[2]) && (a.addr[3] == b.addr[3])/* && (a.port == b.port)*/;
}

inline bool operator!= (const t_XPAddress& a,const t_XPAddress& b)
{
	return !(a == b);
}

typedef struct s_XPSocket t_XPSocket;

typedef struct s_XPSocket {
	int type;										/**< Type of the socket */
	int state;										/**< State of the socket */
	int total_sent;									/**< Total amount of data sent in bytes from this socket */
	int total_received;								/**< Total amount of data received in bytes to this socket */
	int total_accepts;								/**< Total number of accepts this socket has processed if it is a listen socket */
	int total_connects;								/**< Total number of connects for this socket */
	int urgent;										/**< Flagged with TRUE if it is an urgent socket */
	t_XPAddress addr;								/**< Address of external connection for this socket */
	int wins_socket;								/**< For OS socket identifier/handle */
	t_XPSocket *loopback_socket;					/**< Pointer to the internal loopback socket if the socket is in local loopback mode */
	char *loopback_recvbuffer;						/**< Pointer to the receive buffer */
	int loopback_recvpos;							/**< Write position in the receive buffer */
	int	loopback_readpos;							/**< Read position in the receive buffer */
	int loopback_pending;							/**< Pending */
	int port;										/**< The port number */
	} t_XPSocket;									/**< The socket structure */

typedef struct s_XPErrorTranslate {
	int error;										/**< Error number */
	char *text;										/**< Short error text */
	char *longtext;									/**< Long error text */
	} t_XPErrorTranslate;							/**< A structure to use for storing errors */

/* Macros to make life easier */
#define XPSOCK_PARANOID(x)	if ((x) == NULL || (x) == pXPSOCK_EERROR)\
	{\
		asocket_lasterror = XPSOCK_ENOTASOCKET;\
		return XPSOCK_EERROR;\
	}


#define XPSOCK_S_PARANOID(x)	if ((x) == NULL || (x) == pXPSOCK_EERROR)\
	{\
		asocket_lasterror = XPSOCK_ENOTASOCKET;\
		return (t_XPSocket *) XPSOCK_EERROR;\
	}

} // namespace RNXPSockets

// Return error values are either pXPSOCK_EERROR, XPSOCK_EERROR or pXPSOCK_EOK

/* All the global functions */
extern int XPSock_Init(void);
extern int XPSock_Quit(void);
extern RNReplicaNet::t_XPSocket *XPSock_Create(void);
extern int XPSock_SetSendBuffer(RNReplicaNet::t_XPSocket * /*socket*/,int /*size*/);
extern int XPSock_SetRecvBuffer(RNReplicaNet::t_XPSocket * /*socket*/,int /*size*/);
extern int XPSock_Connect(RNReplicaNet::t_XPSocket * /*socket*/,const RNReplicaNet::t_XPAddress * /*addr*/);
extern int XPSock_Send(RNReplicaNet::t_XPSocket * /*socket*/,const char * /*data*/,const int /*len*/);
extern int XPSock_Recv(RNReplicaNet::t_XPSocket * /*socket*/,char * /*data*/,int /*maxlen*/,int /*flag*/);
extern int XPSock_Close(RNReplicaNet::t_XPSocket * /*socket*/);
extern int XPSock_Listen(RNReplicaNet::t_XPSocket * /*socket*/,int /*port*/);
extern RNReplicaNet::t_XPSocket *XPSock_Accept(RNReplicaNet::t_XPSocket * /*socket*/);

extern int XPSock_GetAddress(RNReplicaNet::t_XPSocket * /*socket*/,RNReplicaNet::t_XPAddress * /*addr*/);
extern int XPSock_GetPeerAddress(RNReplicaNet::t_XPSocket * /*socket*/,RNReplicaNet::t_XPAddress * /*addr*/);

extern int XPSock_UrgentSetRawMode(RNReplicaNet::t_XPSocket * /*socket*/,bool /*RawMode*/);
extern RNReplicaNet::t_XPSocket *XPSock_UrgentCreate(int /*port*/);
extern int XPSock_UrgentMakeBroadcast(RNReplicaNet::t_XPSocket * /*socket*/);
extern int XPSock_ProcessACKS(void);
extern int XPSock_UrgentSendWithACK(RNReplicaNet::t_XPSocket * /*socket*/,const char * /*data*/,int /*len*/,const RNReplicaNet::t_XPAddress * /*addr*/);
extern int XPSock_UrgentSend(RNReplicaNet::t_XPSocket * /*socket*/,const char * /*data*/,int /*len*/,const RNReplicaNet::t_XPAddress * /*addr*/);
extern int XPSock_UrgentRecv(RNReplicaNet::t_XPSocket * /*socket*/,char *const /*data*/,int /*maxlen*/,int /*flag*/,RNReplicaNet::t_XPAddress *const /*addr*/);
extern int XPSock_UrgentListen(RNReplicaNet::t_XPSocket * /*socket*/);

extern char *XPSock_TranslateError(int /*errno*/);
extern char *XPSock_TranslateErrorLong(int /*errno*/);

extern int XPSock_GetHostName(char * /*name*/,int /*name_len*/);
extern int XPSock_GetHostAddress(RNReplicaNet::XPAddress * /*addr*/);
extern int XPSock_Resolve(RNReplicaNet::t_XPAddress * /*resolve*/,const char * /*input*/);

extern int XPSock_UtilityGetFragment(char * /*buffer*/,int /*maxlen*/);
extern int XPSock_UtilityGetInt(char * /*buffer*/,int /*offset*/);
extern void XPSock_UtilitySetInt(char * /*buffer*/,int /*offset*/,int /*value*/);
extern short XPSock_UtilityGetShort(char * /*buffer*/,int /*offset*/);
extern void XPSock_UtilitySetShort(char * /*buffer*/,int /*offset*/,short /*value*/);

extern int XPSock_UtilityChecksum(char * /*buffer*/,int /*length*/);

extern void XPSock_Poll(void);

extern void XPSock_SentHistogram(unsigned int * /*classes*/,int /*numClasses*/);
extern void XPSock_RecvHistogram(unsigned int * /*classes*/,int /*numClasses*/);

extern int XPSock_GetHardwareStatus(void);

// The global variables
extern int asocket_lasterror;						/**< This contains the last error value returned from any of these functions */

/* Extern the counters */
extern int	asocket_packets_got;					/**< Total number of packets received */
extern int	asocket_packets_sent;					/**< Total number of packets sent */
extern int	asocket_packets_got_size;				/**< Total byte size of packets received */
extern int	asocket_packets_sent_size;				/**< Total byte size of packets sent */

// Only use these if you know exactly what you are doing
extern bool gXPSock_PacketLossEmulation;
extern float gXPSock_PacketLossEmulationPercent;
extern bool gXPSock_PacketLatencyEmulation;
extern float gXPSock_PacketLatencyEmulationBase;
extern float gXPSock_PacketLatencyEmulationJitter;

#endif

