/* START_LICENSE_HEADER

Copyright (C) 2000 Martin Piper, original design and program code
Copyright (C) 2001-2005 Replica Software

This program file is copyright (C) Replica Software and can only be used under license.
For more information visit: http://www.replicanet.com/
Or email: info@replicanet.com

END_LICENSE_HEADER */
#ifndef __XPSTREAM_H__
#define __XPSTREAM_H__

#include "XPSocketClass.h"

namespace RNReplicaNet
{

/**
 * An implementation enabling a program to send packets via the sockets interface and not lose packet integrity by the transport medium fragmenting packets
 * Most member functions return an XPSOCK_EERROR code.
 * XPSOCK_EOK indicates no error. XPSOCK_EWOULDBLOCK is a non-fatal error and means the socket operation would have blocked the current thread at this point.
 * If XPSOCK_EERROR is returned the user should use XPSocket::GetLastError() to find out the actual error code
 * \todo The XPStream relies too heavily on the underlying XPSocket API. Consider separating this API from the socket base class
 */
class XPStream : public XPSocket
{
public:
	/**
	 * The ctor for an XPStream
	 * At this point no connection is made for the stream
	 */
	XPStream();

	/**
	 * The dtor for an XPStream
	 * This cleans up everything for this stream
	 */
	virtual ~XPStream();

	/**
	 * Creates a XPStream
	 * \return returns an XPSOCK_EERROR code
	 */
	int Create();

	/**
	 * Closes a XPStream
	 * \return returns an XPSOCK_EERROR code
	 */
	int Close();

	/**
	 * Accepts any pending connections for this stream. This should be used with the Listen() member function defined in the XPSocket base class
	 * \return returns NULL if no connection was pending or returns the pointer to the new connected XPStream or pcXPSOCK_EERROR if an error occured
	 */
	XPStream *Accept(void);

	/**
	 * Sends a packet from the connected stream
	 * \param data the pointer to the data to send
	 * \param len the length of the data to send
	 * \return returns an XPSOCK_EERROR code
	 */
	int Send(const char *data,const int len);

	/**
	 * Tries to recv a packet from the stream
	 * \param data the pointer to the data buffer to receive the packet
	 * \param maxlen the maximum length of the data buffer
	 * \return returns the length of the data received or 0 for no data pending or an XPSOCK_EERROR code.
	 */
	int Recv(char *const data,const int maxlen);

	/**
	 * Returns the maximum size for a packet
	 * \return the maximum size for a data packet for this stream
	 */
	int GetMaxPacketSize(void)	{return XPSOCK_MAXPACKETSIZE-2;};	/* Why -2 because each stream packet uses 2 bytes to describe the original length */

private:
	char *mTempBuffer;				/**< A temporary buffer that is used to construct outbound or inbound packets and general data shuffling. Each buffer need one to be thread safe. */
	char *mBuffer;					/**< A buffer that holds incoming packets and merges them until whole packets are available */
	int mPacketBlocks;				/**< The number of maximum size packets that are held in this buffer */
	int mBufferEndPoint;			/**< A counter for the used end point of the buffer */
	int mBufferStartPoint;			/**< A counter that holds the start point of the next packet */
	int mBufferSize;				/**< The size of the stream buffer */
};

} // namespace RNXPSockets

#endif
